@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2019 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -17,13 +17,12 @@ import java.util.Arrays;
 import java.util.HashMap;
 import org.h2.api.ErrorCode;
 import org.h2.engine.Database;
-import org.h2.engine.SysProperties;
 import org.h2.jdbc.JdbcConnection;
 import org.h2.message.DbException;
 import org.h2.tools.CompressTool;
 import org.h2.util.IOUtils;
 import org.h2.util.MathUtils;
-import org.h2.util.New;
+import org.h2.util.Utils;
 import org.h2.value.Value;
 import org.h2.value.ValueLobDb;
 
@@ -140,10 +139,10 @@ public class LobStorageBackend implements LobStorageInterface {
                 }
                 if (create) {
                     stat.execute("CREATE CACHED TABLE IF NOT EXISTS " + LOBS +
-                            "(ID BIGINT PRIMARY KEY, BYTE_COUNT BIGINT, TABLE INT) HIDDEN");
+                            "(ID BIGINT PRIMARY KEY, BYTE_COUNT BIGINT, `TABLE` INT) HIDDEN");
                     stat.execute("CREATE INDEX IF NOT EXISTS " +
                             "INFORMATION_SCHEMA.INDEX_LOB_TABLE ON " +
-                            LOBS + "(TABLE)");
+                            LOBS + "(`TABLE`)");
                     stat.execute("CREATE CACHED TABLE IF NOT EXISTS " + LOB_MAP +
                             "(LOB BIGINT, SEQ INT, POS BIGINT, HASH INT, " +
                             "BLOCK BIGINT, PRIMARY KEY(LOB, SEQ)) HIDDEN");
@@ -192,7 +191,7 @@ public class LobStorageBackend implements LobStorageInterface {
     public void removeAllForTable(int tableId) {
         init();
         try {
-            String sql = "SELECT ID FROM " + LOBS + " WHERE TABLE = ?";
+            String sql = "SELECT ID FROM " + LOBS + " WHERE `TABLE` = ?";
             PreparedStatement prep = prepare(sql);
             prep.setInt(1, tableId);
             ResultSet rs = prep.executeQuery();
@@ -226,9 +225,8 @@ public class LobStorageBackend implements LobStorageInterface {
                 prep.setLong(1, block);
                 ResultSet rs = prep.executeQuery();
                 if (!rs.next()) {
-                    throw DbException.get(ErrorCode.IO_EXCEPTION_1,
-                            "Missing lob entry, block: " + block)
-                            .getSQLException();
+                    throw DbException.getJdbcSQLException(ErrorCode.IO_EXCEPTION_1,
+                            "Missing lob entry, block: " + block);
                 }
                 int compressed = rs.getInt(1);
                 byte[] buffer = rs.getBytes(2);
@@ -248,11 +246,7 @@ public class LobStorageBackend implements LobStorageInterface {
      * @return the prepared statement
      */
     PreparedStatement prepare(String sql) throws SQLException {
-        if (SysProperties.CHECK2) {
-            if (!Thread.holdsLock(database)) {
-                throw DbException.throwInternalError();
-            }
-        }
+        assert Thread.holdsLock(database);
         PreparedStatement prep = prepared.remove(sql);
         if (prep == null) {
             prep = conn.prepareStatement(sql);
@@ -267,11 +261,7 @@ public class LobStorageBackend implements LobStorageInterface {
      * @param prep the prepared statement
      */
     void reuse(String sql, PreparedStatement prep) {
-        if (SysProperties.CHECK2) {
-            if (!Thread.holdsLock(database)) {
-                throw DbException.throwInternalError();
-            }
-        }
+        assert Thread.holdsLock(database);
         prepared.put(sql, prep);
     }
 
@@ -293,7 +283,7 @@ public class LobStorageBackend implements LobStorageInterface {
                     prep.setLong(1, lobId);
                     prep.setLong(2, lobId);
                     ResultSet rs = prep.executeQuery();
-                    ArrayList<Long> blocks = New.arrayList();
+                    ArrayList<Long> blocks = Utils.newSmallArrayList();
                     while (rs.next()) {
                         blocks.add(rs.getLong(1));
                         int hash = rs.getInt(2);
@@ -424,7 +414,7 @@ public class LobStorageBackend implements LobStorageInterface {
         synchronized (database) {
             synchronized (conn.getSession()) {
                 String sql = "INSERT INTO " + LOBS +
-                        "(ID, BYTE_COUNT, TABLE) VALUES(?, ?, ?)";
+                        "(ID, BYTE_COUNT, `TABLE`) VALUES(?, ?, ?)";
                 PreparedStatement prep = prepare(sql);
                 prep.setLong(1, lobId);
                 prep.setLong(2, byteCount);
@@ -444,7 +434,7 @@ public class LobStorageBackend implements LobStorageInterface {
 
     @Override
     public ValueLobDb copyLob(ValueLobDb old, int tableId, long length) {
-        int type = old.getType();
+        int type = old.getValueType();
         long oldLobId = old.getLobId();
         assertNotHolds(conn.getSession());
         // see locking discussion at the top
@@ -466,7 +456,7 @@ public class LobStorageBackend implements LobStorageInterface {
                         reuse(sql, prep);
 
                         sql = "INSERT INTO " + LOBS +
-                                "(ID, BYTE_COUNT, TABLE) " +
+                                "(ID, BYTE_COUNT, `TABLE`) " +
                                 "SELECT ?, BYTE_COUNT, ? FROM " + LOBS +
                                 " WHERE ID = ?";
                         prep = prepare(sql);
@@ -653,8 +643,8 @@ public class LobStorageBackend implements LobStorageInterface {
                 prep.setLong(1, lobId);
                 ResultSet rs = prep.executeQuery();
                 if (!rs.next()) {
-                    throw DbException.get(ErrorCode.IO_EXCEPTION_1,
-                            "Missing lob entry: " + lobId).getSQLException();
+                    throw DbException.getJdbcSQLException(ErrorCode.IO_EXCEPTION_1,
+                            "Missing lob entry: " + lobId);
                 }
                 byteCount = rs.getLong(1);
                 reuse(sql, prep);
@@ -668,8 +658,8 @@ public class LobStorageBackend implements LobStorageInterface {
             rs.next();
             int lobMapCount = rs.getInt(1);
             if (lobMapCount == 0) {
-                throw DbException.get(ErrorCode.IO_EXCEPTION_1,
-                        "Missing lob entry: " + lobId).getSQLException();
+                throw DbException.getJdbcSQLException(ErrorCode.IO_EXCEPTION_1,
+                        "Missing lob entry: " + lobId);
             }
             reuse(sql, prep);
 
