@@ -1,5 +1,5 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2019 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
@@ -30,19 +30,61 @@ import java.util.GregorianCalendar;
 import java.util.UUID;
 
 import org.h2.api.ErrorCode;
+import org.h2.api.Interval;
+import org.h2.api.IntervalQualifier;
 import org.h2.api.Trigger;
 import org.h2.engine.SysProperties;
+import org.h2.message.DbException;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.util.LocalDateTimeUtils;
 import org.h2.util.Task;
 
 /**
  * Tests for the PreparedStatement implementation.
  */
-public class TestPreparedStatement extends TestBase {
+public class TestPreparedStatement extends TestDb {
 
     private static final int LOB_SIZE = 4000, LOB_SIZE_BIG = 512 * 1024;
 
+    /**
+     * {@code java.time.LocalDate#parse(CharSequence)} or {@code null}.
+     */
+    private static final Method LOCAL_DATE_PARSE;
+
+    /**
+     * {@code java.time.LocalTime#parse(CharSequence)} or {@code null}.
+     */
+    private static final Method LOCAL_TIME_PARSE;
+
+    /**
+     * {@code java.time.LocalDateTime#parse(CharSequence)} or {@code null}.
+     */
+    private static final Method LOCAL_DATE_TIME_PARSE;
+
+    /**
+     * {@code java.time.OffsetDateTime#parse(CharSequence)} or {@code null}.
+     */
+    private static final Method OFFSET_DATE_TIME_PARSE;
+
+    static {
+        if (LocalDateTimeUtils.isJava8DateApiPresent()) {
+            try {
+                LOCAL_DATE_PARSE = LocalDateTimeUtils.LOCAL_DATE.getMethod("parse", CharSequence.class);
+                LOCAL_TIME_PARSE = LocalDateTimeUtils.LOCAL_TIME.getMethod("parse", CharSequence.class);
+                LOCAL_DATE_TIME_PARSE = LocalDateTimeUtils.LOCAL_DATE_TIME.getMethod("parse", CharSequence.class);
+                OFFSET_DATE_TIME_PARSE = LocalDateTimeUtils.OFFSET_DATE_TIME.getMethod("parse", CharSequence.class);
+            } catch (NoSuchMethodException e) {
+                throw DbException.convert(e);
+            }
+        } else {
+            LOCAL_DATE_PARSE = null;
+            LOCAL_TIME_PARSE = null;
+            LOCAL_DATE_TIME_PARSE = null;
+            OFFSET_DATE_TIME_PARSE = null;
+        }
+    }
+
     /**
      * Run just this test.
      *
@@ -52,6 +94,62 @@ public class TestPreparedStatement extends TestBase {
         TestBase.createCaller().init().test();
     }
 
+    /**
+     * Parses an ISO date string into a java.time.LocalDate.
+     *
+     * @param text the ISO date string
+     * @return the java.time.LocalDate instance
+     */
+    public static Object parseLocalDate(CharSequence text) {
+        try {
+            return LOCAL_DATE_PARSE.invoke(null, text);
+        } catch (IllegalAccessException | InvocationTargetException e) {
+            throw new IllegalArgumentException("error when parsing text '" + text + "'", e);
+        }
+    }
+
+    /**
+     * Parses an ISO time string into a java.time.LocalTime.
+     *
+     * @param text the ISO time string
+     * @return the java.time.LocalTime instance
+     */
+    public static Object parseLocalTime(CharSequence text) {
+        try {
+            return LOCAL_TIME_PARSE.invoke(null, text);
+        } catch (IllegalAccessException | InvocationTargetException e) {
+            throw new IllegalArgumentException("error when parsing text '" + text + "'", e);
+        }
+    }
+
+    /**
+     * Parses an ISO date string into a java.time.LocalDateTime.
+     *
+     * @param text the ISO date string
+     * @return the java.time.LocalDateTime instance
+     */
+    public static Object parseLocalDateTime(CharSequence text) {
+        try {
+            return LOCAL_DATE_TIME_PARSE.invoke(null, text);
+        } catch (IllegalAccessException | InvocationTargetException e) {
+            throw new IllegalArgumentException("error when parsing text '" + text + "'", e);
+        }
+    }
+
+    /**
+     * Parses an ISO date string into a java.time.OffsetDateTime.
+     *
+     * @param text the ISO date string
+     * @return the java.time.OffsetDateTime instance
+     */
+    public static Object parseOffsetDateTime(CharSequence text) {
+        try {
+            return OFFSET_DATE_TIME_PARSE.invoke(null, text);
+        } catch (IllegalAccessException | InvocationTargetException e) {
+            throw new IllegalArgumentException("error when parsing text '" + text + "'", e);
+        }
+    }
+
     @Override
     public void test() throws Exception {
         deleteDb("preparedStatement");
@@ -84,6 +182,8 @@ public class TestPreparedStatement extends TestBase {
         testDateTime8(conn);
         testOffsetDateTime8(conn);
         testInstant8(conn);
+        testInterval(conn);
+        testInterval8(conn);
         testArray(conn);
         testSetObject(conn);
         testPreparedSubquery(conn);
@@ -98,6 +198,8 @@ public class TestPreparedStatement extends TestBase {
         testParameterMetaData(conn);
         testColumnMetaDataWithEquals(conn);
         testColumnMetaDataWithIn(conn);
+        testValueResultSet(conn);
+        testMultipleStatements(conn);
         conn.close();
         testPreparedStatementWithLiteralsNone();
         testPreparedStatementWithIndexedParameterAndLiteralsNone();
@@ -152,8 +254,6 @@ public class TestPreparedStatement extends TestBase {
 
         ParameterMetaData meta = prep.getParameterMetaData();
         assertTrue(meta.toString(), meta.toString().endsWith("parameterCount=1"));
-        assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, conn).
-                createSQLXML();
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, conn).
                 createStruct("Integer", new Object[0]);
     }
@@ -404,7 +504,7 @@ public class TestPreparedStatement extends TestBase {
         Thread.sleep(100);
         prep.cancel();
         SQLException e = (SQLException) t.getException();
-        assertTrue(e != null);
+        assertNotNull(e);
         assertEquals(ErrorCode.STATEMENT_WAS_CANCELED, e.getErrorCode());
         prep.setInt(1, 1);
         prep.setInt(2, 1);
@@ -476,7 +576,7 @@ public class TestPreparedStatement extends TestBase {
             assertEquals(goodSizes[i], rs.getString(1));
             assertEquals(i, rs.getInt(1));
             Object o = rs.getObject(1);
-            assertEquals(Integer.class, o.getClass());
+            assertEquals(String.class, o.getClass());
         }
 
         for (int i = 0; i < goodSizes.length; i++) {
@@ -618,7 +718,7 @@ public class TestPreparedStatement extends TestBase {
         rs.next();
         Object o = rs.getObject(2);
         assertTrue(o instanceof byte[]);
-        assertTrue(rs.getObject(3) == null);
+        assertNull(rs.getObject(3));
         rs.next();
         o = rs.getObject(2);
         assertTrue(o instanceof byte[]);
@@ -644,14 +744,14 @@ public class TestPreparedStatement extends TestBase {
             return;
         }
         PreparedStatement prep = conn.prepareStatement("SELECT ?");
-        Object localDate = LocalDateTimeUtils.parseLocalDate("2001-02-03");
+        Object localDate = parseLocalDate("2001-02-03");
         prep.setObject(1, localDate);
         ResultSet rs = prep.executeQuery();
         rs.next();
         Object localDate2 = rs.getObject(1, LocalDateTimeUtils.LOCAL_DATE);
         assertEquals(localDate, localDate2);
         rs.close();
-        localDate = LocalDateTimeUtils.parseLocalDate("-0509-01-01");
+        localDate = parseLocalDate("-0509-01-01");
         prep.setObject(1, localDate);
         rs = prep.executeQuery();
         rs.next();
@@ -666,31 +766,31 @@ public class TestPreparedStatement extends TestBase {
         rs = prep.executeQuery();
         rs.next();
         localDate2 = rs.getObject(1, LocalDateTimeUtils.LOCAL_DATE);
-        assertEquals(LocalDateTimeUtils.parseLocalDate("1500-03-01"), localDate2);
+        assertEquals(parseLocalDate("1500-03-01"), localDate2);
         rs.close();
         prep.setString(1, "1400-02-29");
         rs = prep.executeQuery();
         rs.next();
         localDate2 = rs.getObject(1, LocalDateTimeUtils.LOCAL_DATE);
-        assertEquals(LocalDateTimeUtils.parseLocalDate("1400-03-01"), localDate2);
+        assertEquals(parseLocalDate("1400-03-01"), localDate2);
         rs.close();
         prep.setString(1, "1300-02-29");
         rs = prep.executeQuery();
         rs.next();
         localDate2 = rs.getObject(1, LocalDateTimeUtils.LOCAL_DATE);
-        assertEquals(LocalDateTimeUtils.parseLocalDate("1300-03-01"), localDate2);
+        assertEquals(parseLocalDate("1300-03-01"), localDate2);
         rs.close();
         prep.setString(1, "-0100-02-29");
         rs = prep.executeQuery();
         rs.next();
         localDate2 = rs.getObject(1, LocalDateTimeUtils.LOCAL_DATE);
-        assertEquals(LocalDateTimeUtils.parseLocalDate("-0100-03-01"), localDate2);
+        assertEquals(parseLocalDate("-0100-03-01"), localDate2);
         rs.close();
         /*
          * Check that date that doesn't exist in traditional calendar can be set and
          * read with LocalDate and can be read with getDate() as a next date.
          */
-        localDate = LocalDateTimeUtils.parseLocalDate("1582-10-05");
+        localDate = parseLocalDate("1582-10-05");
         prep.setObject(1, localDate);
         rs = prep.executeQuery();
         rs.next();
@@ -719,14 +819,14 @@ public class TestPreparedStatement extends TestBase {
             return;
         }
         PreparedStatement prep = conn.prepareStatement("SELECT ?");
-        Object localTime = LocalDateTimeUtils.parseLocalTime("04:05:06");
+        Object localTime = parseLocalTime("04:05:06");
         prep.setObject(1, localTime);
         ResultSet rs = prep.executeQuery();
         rs.next();
         Object localTime2 = rs.getObject(1, LocalDateTimeUtils.LOCAL_TIME);
         assertEquals(localTime, localTime2);
         rs.close();
-        localTime = LocalDateTimeUtils.parseLocalTime("04:05:06.123456789");
+        localTime = parseLocalTime("04:05:06.123456789");
         prep.setObject(1, localTime);
         rs = prep.executeQuery();
         rs.next();
@@ -740,7 +840,7 @@ public class TestPreparedStatement extends TestBase {
             return;
         }
         PreparedStatement prep = conn.prepareStatement("SELECT ?");
-        Object localDateTime = LocalDateTimeUtils.parseLocalDateTime("2001-02-03T04:05:06");
+        Object localDateTime = parseLocalDateTime("2001-02-03T04:05:06");
         prep.setObject(1, localDateTime);
         ResultSet rs = prep.executeQuery();
         rs.next();
@@ -754,8 +854,7 @@ public class TestPreparedStatement extends TestBase {
             return;
         }
         PreparedStatement prep = conn.prepareStatement("SELECT ?");
-        Object offsetDateTime = LocalDateTimeUtils
-                .parseOffsetDateTime("2001-02-03T04:05:06+02:30");
+        Object offsetDateTime = parseOffsetDateTime("2001-02-03T04:05:06+02:30");
         prep.setObject(1, offsetDateTime);
         ResultSet rs = prep.executeQuery();
         rs.next();
@@ -809,6 +908,75 @@ public class TestPreparedStatement extends TestBase {
         rs.close();
     }
 
+    private void testInterval(Connection conn) throws SQLException {
+        PreparedStatement prep = conn.prepareStatement("SELECT ?");
+        Interval interval = new Interval(IntervalQualifier.MINUTE, false, 100, 0);
+        prep.setObject(1, interval);
+        ResultSet rs = prep.executeQuery();
+        rs.next();
+        assertEquals("INTERVAL '100' MINUTE", rs.getString(1));
+        assertEquals(interval, rs.getObject(1));
+        assertEquals(interval, rs.getObject(1, Interval.class));
+    }
+
+    private void testInterval8(Connection conn) throws SQLException {
+        if (!LocalDateTimeUtils.isJava8DateApiPresent()) {
+            return;
+        }
+        PreparedStatement prep = conn.prepareStatement("SELECT ?");
+        testPeriod8(prep, 1, 2, "INTERVAL '1-2' YEAR TO MONTH");
+        testPeriod8(prep, -1, -2, "INTERVAL '-1-2' YEAR TO MONTH");
+        testPeriod8(prep, 1, -8, "INTERVAL '0-4' YEAR TO MONTH", 0, 4);
+        testPeriod8(prep, -1, 8, "INTERVAL '-0-4' YEAR TO MONTH", 0, -4);
+        testPeriod8(prep, 0, 0, "INTERVAL '0-0' YEAR TO MONTH");
+        testPeriod8(prep, 100, 0, "INTERVAL '100' YEAR");
+        testPeriod8(prep, -100, 0, "INTERVAL '-100' YEAR");
+        testPeriod8(prep, 0, 100, "INTERVAL '100' MONTH");
+        testPeriod8(prep, 0, -100, "INTERVAL '-100' MONTH");
+        Object period;
+        try {
+            Method method = LocalDateTimeUtils.PERIOD.getMethod("of", int.class, int.class, int.class);
+            period = method.invoke(null, 0, 0, 1);
+        } catch (ReflectiveOperationException ex) {
+            throw new RuntimeException(ex);
+        }
+        assertThrows(ErrorCode.INVALID_VALUE_2, prep).setObject(1, period);
+        Object duration;
+        try {
+            duration = LocalDateTimeUtils.DURATION.getMethod("ofSeconds", long.class, long.class)
+                    .invoke(null, -4, 900_000_000);
+        } catch (ReflectiveOperationException ex) {
+            throw new RuntimeException(ex);
+        }
+        prep.setObject(1, duration);
+        ResultSet rs = prep.executeQuery();
+        rs.next();
+        assertEquals("INTERVAL '-3.1' SECOND", rs.getString(1));
+        assertEquals(duration, rs.getObject(1, LocalDateTimeUtils.DURATION));
+    }
+
+    private void testPeriod8(PreparedStatement prep, int years, int months, String expectedString)
+            throws SQLException {
+        testPeriod8(prep, years, months, expectedString, years, months);
+    }
+
+    private void testPeriod8(PreparedStatement prep, int years, int months, String expectedString, int expYears,
+            int expMonths) throws SQLException {
+        Object period, expectedPeriod;
+        try {
+            Method method = LocalDateTimeUtils.PERIOD.getMethod("of", int.class, int.class, int.class);
+            period = method.invoke(null, years, months, 0);
+            expectedPeriod = method.invoke(null, expYears, expMonths, 0);
+        } catch (ReflectiveOperationException ex) {
+            throw new RuntimeException(ex);
+        }
+        prep.setObject(1, period);
+        ResultSet rs = prep.executeQuery();
+        rs.next();
+        assertEquals(expectedString, rs.getString(1));
+        assertEquals(expectedPeriod, rs.getObject(1, LocalDateTimeUtils.PERIOD));
+    }
+
     private void testPreparedSubquery(Connection conn) throws SQLException {
         Statement s = conn.createStatement();
         s.executeUpdate("CREATE TABLE TEST(ID IDENTITY, FLAG BIT)");
@@ -1446,9 +1614,9 @@ public class TestPreparedStatement extends TestBase {
         assertEquals(ascii2, rs.getString(3));
 
         assertFalse(rs.next());
-        assertTrue(prep.getWarnings() == null);
+        assertNull(prep.getWarnings());
         prep.clearWarnings();
-        assertTrue(prep.getWarnings() == null);
+        assertNull(prep.getWarnings());
         assertTrue(conn == prep.getConnection());
     }
 
@@ -1507,6 +1675,12 @@ public class TestPreparedStatement extends TestBase {
         anyParameterCheck(ps, values, expected);
         ps = conn.prepareStatement("SELECT ID FROM TEST INNER JOIN TABLE(X INT=?) T ON TEST.VALUE = T.X");
         anyParameterCheck(ps, values, expected);
+        // Test expression IN(UNNEST(?))
+        ps = conn.prepareStatement("SELECT ID FROM TEST WHERE VALUE IN(UNNEST(?))");
+        assertThrows(ErrorCode.PARAMETER_NOT_SET_1, ps).executeQuery();
+        anyParameterCheck(ps, values, expected);
+        anyParameterCheck(ps, 300, new int[] {30});
+        anyParameterCheck(ps, -5, new int[0]);
         // Test expression = ANY(?)
         ps = conn.prepareStatement("SELECT ID FROM TEST WHERE VALUE = ANY(?)");
         assertThrows(ErrorCode.PARAMETER_NOT_SET_1, ps).executeQuery();
@@ -1534,12 +1708,12 @@ public class TestPreparedStatement extends TestBase {
             java.math.BigDecimal x = rs.getBigDecimal(1);
             trace("v=" + v + " x=" + x);
             if (v == null) {
-                assertTrue(x == null);
+                assertNull(x);
             } else {
                 assertTrue(x.compareTo(new java.math.BigDecimal(v)) == 0);
             }
         }
-        assertTrue(!rs.next());
+        assertFalse(rs.next());
     }
 
     private void testColumnMetaDataWithEquals(Connection conn)
@@ -1571,4 +1745,45 @@ public class TestPreparedStatement extends TestBase {
                 ps.getParameterMetaData().getParameterType(1));
         stmt.execute("DROP TABLE TEST");
     }
+
+    private void testValueResultSet(Connection conn) throws SQLException {
+        for (int i = 0; i < 2; i++) {
+            try (PreparedStatement stmt = conn.prepareStatement("SELECT TABLE(X INT = (1))")) {
+                ResultSet rs = stmt.executeQuery();
+                while (rs.next()) {
+                    try (ResultSet rs2 = (ResultSet) rs.getObject(1)) {
+                        assertEquals(1, rs2.getMetaData().getColumnCount());
+                    }
+                }
+            }
+        }
+    }
+
+    private void testMultipleStatements(Connection conn) throws SQLException {
+        assertThrows(ErrorCode.CANNOT_MIX_INDEXED_AND_UNINDEXED_PARAMS, conn).prepareStatement("SELECT ?; SELECT ?1");
+        assertThrows(ErrorCode.CANNOT_MIX_INDEXED_AND_UNINDEXED_PARAMS, conn).prepareStatement("SELECT ?1; SELECT ?");
+        Statement stmt = conn.createStatement();
+        stmt.execute("CREATE TABLE TEST (ID IDENTITY, V INT)");
+        PreparedStatement ps = conn.prepareStatement("INSERT INTO TEST(V) VALUES ?; INSERT INTO TEST(V) VALUES ?");
+        ps.setInt(1, 1);
+        ps.setInt(2, 2);
+        ps.executeUpdate();
+        ps = conn.prepareStatement("INSERT INTO TEST(V) VALUES ?2; INSERT INTO TEST(V) VALUES ?1;");
+        ps.setInt(1, 3);
+        ps.setInt(2, 4);
+        ps.executeUpdate();
+        try (ResultSet rs = stmt.executeQuery("SELECT V FROM TEST ORDER BY ID")) {
+            assertTrue(rs.next());
+            assertEquals(1, rs.getInt(1));
+            assertTrue(rs.next());
+            assertEquals(2, rs.getInt(1));
+            assertTrue(rs.next());
+            assertEquals(4, rs.getInt(1));
+            assertTrue(rs.next());
+            assertEquals(3, rs.getInt(1));
+            assertFalse(rs.next());
+        }
+        stmt.execute("DROP TABLE TEST");
+    }
+
 }
