@@ -1,62 +1,97 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * Copyright 2004-2019 H2 Group. Multiple-Licensed under the MPL 2.0,
  * and the EPL 1.0 (http://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.scripts;
 
+import java.io.File;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.LineNumberReader;
 import java.io.PrintStream;
+import java.io.RandomAccessFile;
+import java.lang.reflect.Field;
+import java.lang.reflect.Modifier;
+import java.nio.charset.StandardCharsets;
 import java.sql.Connection;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.sql.Statement;
+import java.util.ArrayDeque;
 import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.Map;
 import java.util.Random;
 
+import org.h2.api.ErrorCode;
+import org.h2.command.CommandContainer;
+import org.h2.command.CommandInterface;
+import org.h2.command.Prepared;
+import org.h2.command.dml.Query;
 import org.h2.engine.SysProperties;
+import org.h2.jdbc.JdbcConnection;
+import org.h2.jdbc.JdbcPreparedStatement;
 import org.h2.test.TestAll;
 import org.h2.test.TestBase;
-import org.h2.util.New;
+import org.h2.test.TestDb;
 import org.h2.util.StringUtils;
 
 /**
  * This test runs a SQL script file and compares the output with the expected
  * output.
  */
-public class TestScript extends TestBase {
+public class TestScript extends TestDb {
 
     private static final String BASE_DIR = "org/h2/test/scripts/";
 
+    private static final boolean FIX_OUTPUT = false;
+
+    private static final Field COMMAND;
+
+    private static final Field PREPARED;
+
+    private static boolean CHECK_ORDERING;
+
     /** If set to true, the test will exit at the first failure. */
     private boolean failFast;
-    private final ArrayList<String> statements = New.arrayList();
+    /** If set to a value the test will add all executed statements to this list */
+    private ArrayList<String> statements;
 
     private boolean reconnectOften;
     private Connection conn;
     private Statement stat;
     private String fileName;
     private LineNumberReader in;
-    private int outputLineNo;
     private PrintStream out;
-    private final ArrayList<String[]> result = New.arrayList();
-    private String putBack;
+    private final ArrayList<String[]> result = new ArrayList<>();
+    private final ArrayDeque<String> putBack = new ArrayDeque<>();
     private StringBuilder errors;
 
     private Random random = new Random(1);
 
+    static {
+        try {
+            COMMAND = JdbcPreparedStatement.class.getDeclaredField("command");
+            COMMAND.setAccessible(true);
+            PREPARED = CommandContainer.class.getDeclaredField("prepared");
+            PREPARED.setAccessible(true);
+        } catch (ReflectiveOperationException e) {
+            throw new RuntimeException(e);
+        }
+    }
+
     /**
      * Run just this test.
      *
      * @param a ignored
      */
     public static void main(String... a) throws Exception {
+        CHECK_ORDERING = true;
         TestBase.createCaller().init().test();
     }
 
@@ -68,27 +103,45 @@ public class TestScript extends TestBase {
      */
     public ArrayList<String> getAllStatements(TestAll conf) throws Exception {
         config = conf;
-        if (statements.isEmpty()) {
+        ArrayList<String> result = new ArrayList<>(4000);
+        try {
+            statements = result;
             test();
+        } finally {
+            this.statements = null;
         }
-        return statements;
+        return result;
+    }
+
+    @Override
+    public boolean isEnabled() {
+        if (config.networked && config.big) {
+            return false;
+        }
+        return true;
     }
 
     @Override
     public void test() throws Exception {
-        if (config.networked && config.big) {
-            return;
-        }
         reconnectOften = !config.memory && config.big;
 
         testScript("testScript.sql");
+        if (!config.memory && !config.big && !config.networked) {
+            testScript("testSimple.sql");
+        }
+        testScript("comments.sql");
         testScript("derived-column-names.sql");
+        testScript("distinct.sql");
+        testScript("dual.sql");
+        testScript("indexes.sql");
         testScript("information_schema.sql");
         testScript("joins.sql");
         testScript("range_table.sql");
         testScript("altertable-index-reuse.sql");
+        testScript("altertable-fk.sql");
         testScript("default-and-on_update.sql");
         testScript("query-optimisations.sql");
+        testScript("window.sql");
         String decimal2;
         if (SysProperties.BIG_DECIMAL_IS_DECIMAL) {
             decimal2 = "decimal_decimal";
@@ -98,27 +151,32 @@ public class TestScript extends TestBase {
 
         for (String s : new String[] { "array", "bigint", "binary", "blob",
                 "boolean", "char", "clob", "date", "decimal", decimal2, "double", "enum",
-                "geometry", "identity", "int", "other", "real", "smallint",
+                "geometry", "identity", "int", "interval", "other", "real", "row", "smallint",
                 "time", "timestamp-with-timezone", "timestamp", "tinyint",
                 "uuid", "varchar", "varchar-ignorecase" }) {
             testScript("datatypes/" + s + ".sql");
         }
-        for (String s : new String[] { "alterTableAdd", "alterTableDropColumn", "createView", "createTable",
-                "dropSchema" }) {
+        for (String s : new String[] { "alterTableAdd", "alterTableAlterColumn", "alterTableDropColumn",
+                "alterTableRename", "createAlias", "createSequence", "createSynonym", "createTable", "createTrigger",
+                "createView", "dropDomain", "dropIndex", "dropSchema", "truncateTable" }) {
             testScript("ddl/" + s + ".sql");
         }
-        for (String s : new String[] { "insertIgnore", "mergeUsing", "script", "with" }) {
+        for (String s : new String[] { "delete", "error_reporting", "insert", "insertIgnore", "merge", "mergeUsing",
+                "replace", "script", "select", "show", "table", "update", "values", "with" }) {
             testScript("dml/" + s + ".sql");
         }
-        for (String s : new String[] { "avg", "bit-and", "bit-or", "count",
-                "group-concat", "max", "median", "min", "selectivity", "stddev-pop",
-                "stddev-samp", "sum", "var-pop", "var-samp", "array-agg" }) {
+        for (String s : new String[] { "help" }) {
+            testScript("other/" + s + ".sql");
+        }
+        for (String s : new String[] { "any", "array-agg", "avg", "bit-and", "bit-or", "count", "envelope",
+                "every", "histogram", "listagg", "max", "min", "mode", "percentile", "rank", "selectivity",
+                "stddev-pop", "stddev-samp", "sum", "var-pop", "var-samp" }) {
             testScript("functions/aggregate/" + s + ".sql");
         }
         for (String s : new String[] { "abs", "acos", "asin", "atan", "atan2",
                 "bitand", "bitget", "bitor", "bitxor", "ceil", "compress",
                 "cos", "cosh", "cot", "decrypt", "degrees", "encrypt", "exp",
-                "expand", "floor", "hash", "length", "log", "mod", "pi",
+                "expand", "floor", "hash", "length", "log", "mod", "ora-hash", "pi",
                 "power", "radians", "rand", "random-uuid", "round",
                 "roundmagic", "secure-rand", "sign", "sin", "sinh", "sqrt",
                 "tan", "tanh", "truncate", "zero" }) {
@@ -135,15 +193,15 @@ public class TestScript extends TestBase {
                 "xmlnode", "xmlstartdoc", "xmltext" }) {
             testScript("functions/string/" + s + ".sql");
         }
-        for (String s : new String[] { "array-contains", "array-get",
-                "array-length", "autocommit", "cancel-session", "casewhen",
+        for (String s : new String[] { "array-cat", "array-contains", "array-get",
+                "array-length","array-slice", "autocommit", "cancel-session", "casewhen",
                 "cast", "coalesce", "convert", "csvread", "csvwrite", "currval",
                 "database-path", "database", "decode", "disk-space-used",
                 "file-read", "file-write", "greatest", "h2version", "identity",
                 "ifnull", "least", "link-schema", "lock-mode", "lock-timeout",
                 "memory-free", "memory-used", "nextval", "nullif", "nvl2",
                 "readonly", "rownum", "schema", "scope-identity", "session-id",
-                "set", "table", "transaction-id", "truncate-value", "user" }) {
+                "set", "table", "transaction-id", "truncate-value", "unnest", "user" }) {
             testScript("functions/system/" + s + ".sql");
         }
         for (String s : new String[] { "add_months", "current_date", "current_timestamp",
@@ -153,6 +211,9 @@ public class TestScript extends TestBase {
                 "parsedatetime", "quarter", "second", "truncate", "week", "year", "date_trunc" }) {
             testScript("functions/timeanddate/" + s + ".sql");
         }
+        for (String s : new String[] { "lead", "nth_value", "ntile", "ratio_to_report", "row_number" }) {
+            testScript("functions/window/" + s + ".sql");
+        }
 
         deleteDb("script");
         System.out.flush();
@@ -167,14 +228,24 @@ public class TestScript extends TestBase {
         stat = null;
         fileName = null;
         in = null;
-        outputLineNo = 0;
         out = null;
         result.clear();
-        putBack = null;
+        putBack.clear();
         errors = null;
 
-        println("Running commands in " + scriptFileName);
-        final String outFile = "test.out.txt";
+        if (statements == null) {
+            println("Running commands in " + scriptFileName);
+        }
+        String outFile;
+        if (FIX_OUTPUT) {
+            outFile = scriptFileName;
+            int idx = outFile.lastIndexOf('/');
+            if (idx >= 0) {
+                outFile = outFile.substring(idx + 1);
+            }
+        } else {
+            outFile = "test.out.txt";
+        }
         conn = getConnection("script");
         stat = conn.createStatement();
         out = new PrintStream(new FileOutputStream(outFile));
@@ -182,28 +253,96 @@ public class TestScript extends TestBase {
         testFile(BASE_DIR + scriptFileName);
         conn.close();
         out.close();
+        if (FIX_OUTPUT) {
+            File file = new File(outFile);
+            // If there are two trailing newline characters remove one
+            try (RandomAccessFile r = new RandomAccessFile(file, "rw")) {
+                byte[] separator = System.lineSeparator().getBytes(StandardCharsets.ISO_8859_1);
+                int separatorLength = separator.length;
+                long length = r.length() - (separatorLength * 2);
+                truncate: if (length >= 0) {
+                    r.seek(length);
+                    for (int i = 0; i < 2; i++) {
+                        for (int j = 0; j < separatorLength; j++) {
+                            if (r.readByte() != separator[j]) {
+                                break truncate;
+                            }
+                        }
+                    }
+                    r.setLength(length + separatorLength);
+                }
+            }
+            file.renameTo(new File("h2/src/test/org/h2/test/scripts/" + scriptFileName));
+            return;
+        }
         if (errors.length() > 0) {
             throw new Exception("errors in " + scriptFileName + " found");
         }
-        // new File(outFile).delete();
     }
 
     private String readLine() throws IOException {
-        if (putBack != null) {
-            String s = putBack;
-            putBack = null;
-            return s;
-        }
-        while (true) {
-            String s = in.readLine();
-            if (s == null) {
-                return s;
+        String s = putBack.pollFirst();
+        return s != null ? s : readNextLine();
+    }
+
+    private String readNextLine() throws IOException {
+        String s;
+        boolean comment = false;
+        while ((s = in.readLine()) != null) {
+            if (s.startsWith("#")) {
+                int end = s.indexOf('#', 1);
+                if (end < 3) {
+                    fail("Bad line \"" + s + '\"');
+                }
+                boolean val;
+                switch (s.charAt(1)) {
+                case '+':
+                    val = true;
+                    break;
+                case '-':
+                    val = false;
+                    break;
+                default:
+                    fail("Bad line \"" + s + '\"');
+                    return null;
+                }
+                String flag = s.substring(2, end);
+                s = s.substring(end + 1);
+                switch (flag) {
+                case "mvStore":
+                    if (config.mvStore == val) {
+                        out.print("#" + (val ? '+' : '-') + flag + '#');
+                        break;
+                    } else {
+                        if (FIX_OUTPUT) {
+                            write("#" + (val ? '+' : '-') + flag + '#' + s);
+                        }
+                        continue;
+                    }
+                default:
+                    fail("Unknown flag \"" + flag + '\"');
+                }
+            } else if (s.startsWith("--")) {
+                write(s);
+                comment = true;
+                continue;
             }
-            s = s.trim();
-            if (s.length() > 0) {
-                return s;
+            if (!FIX_OUTPUT) {
+                s = s.trim();
+            }
+            if (!s.isEmpty()) {
+                break;
+            }
+            if (comment) {
+                write("");
+                comment = false;
             }
         }
+        return s;
+    }
+
+    private void putBack(String line) {
+        putBack.addLast(line);
     }
 
     private void testFile(String inFile) throws Exception {
@@ -212,23 +351,46 @@ public class TestScript extends TestBase {
             throw new IOException("could not find " + inFile);
         }
         fileName = inFile;
-        in = new LineNumberReader(new InputStreamReader(is, "Cp1252"));
+        in = new LineNumberReader(new InputStreamReader(is, StandardCharsets.UTF_8));
         StringBuilder buff = new StringBuilder();
-        while (true) {
-            String sql = readLine();
-            if (sql == null) {
-                break;
-            }
+        boolean allowReconnect = true;
+        for (String sql; (sql = readLine()) != null;) {
             if (sql.startsWith("--")) {
                 write(sql);
             } else if (sql.startsWith(">")) {
-                // do nothing
+                addWriteResultError("<command>", sql);
             } else if (sql.endsWith(";")) {
                 write(sql);
                 buff.append(sql, 0, sql.length() - 1);
                 sql = buff.toString();
-                buff = new StringBuilder();
-                process(sql);
+                buff.setLength(0);
+                process(sql, allowReconnect);
+            } else if (sql.startsWith("@")) {
+                if (buff.length() > 0) {
+                    addWriteResultError("<command>", sql);
+                } else {
+                    switch (sql) {
+                    case "@reconnect":
+                        write(sql);
+                        write("");
+                        if (!config.memory) {
+                            reconnect(conn.getAutoCommit());
+                        }
+                        break;
+                    case "@reconnect on":
+                        write(sql);
+                        write("");
+                        allowReconnect = true;
+                        break;
+                    case "@reconnect off":
+                        write(sql);
+                        write("");
+                        allowReconnect = false;
+                        break;
+                    default:
+                        addWriteResultError("<command>", sql);
+                    }
+                }
             } else {
                 write(sql);
                 buff.append(sql);
@@ -251,20 +413,20 @@ public class TestScript extends TestBase {
         return false;
     }
 
-    private void process(String sql) throws Exception {
-        if (reconnectOften) {
-            if (!containsTempTables()) {
+    private void process(String sql, boolean allowReconnect) throws Exception {
+        if (allowReconnect && reconnectOften) {
+            if (!containsTempTables() && ((JdbcConnection) conn).isRegularMode()
+                    && conn.getSchema().equals("PUBLIC")) {
                 boolean autocommit = conn.getAutoCommit();
                 if (autocommit && random.nextInt(10) < 1) {
                     // reconnect 10% of the time
-                    conn.close();
-                    conn = getConnection("script");
-                    conn.setAutoCommit(autocommit);
-                    stat = conn.createStatement();
+                    reconnect(autocommit);
                 }
             }
         }
-        statements.add(sql);
+        if (statements != null) {
+            statements.add(sql);
+        }
         if (sql.indexOf('?') == -1) {
             processStatement(sql);
         } else {
@@ -292,6 +454,13 @@ public class TestScript extends TestBase {
         write("");
     }
 
+    private void reconnect(boolean autocommit) throws SQLException {
+        conn.close();
+        conn = getConnection("script");
+        conn.setAutoCommit(autocommit);
+        stat = conn.createStatement();
+    }
+
     private static void setParameter(PreparedStatement prep, int i, String param)
             throws SQLException {
         if (param.equalsIgnoreCase("null")) {
@@ -309,7 +478,7 @@ public class TestScript extends TestBase {
                 char c = param.charAt(i);
                 if (c == ',') {
                     setParameter(prep, ++index, buff.toString());
-                    buff = new StringBuilder();
+                    buff.setLength(0);
                 } else if (c == '"') {
                     while (true) {
                         c = param.charAt(++i);
@@ -338,10 +507,20 @@ public class TestScript extends TestBase {
 
     private int processStatement(String sql) throws Exception {
         try {
-            if (stat.execute(sql)) {
-                writeResultSet(sql, stat.getResultSet());
+            boolean res;
+            Statement s;
+            if (/* TestScript */ CHECK_ORDERING || /* TestAll */ config.memory && !config.lazy && !config.networked) {
+                PreparedStatement prep = conn.prepareStatement(sql);
+                res = prep.execute();
+                s = prep;
             } else {
-                int count = stat.getUpdateCount();
+                res = stat.execute(sql);
+                s = stat;
+            }
+            if (res) {
+                writeResultSet(sql, s.getResultSet());
+            } else {
+                int count = s.getUpdateCount();
                 writeResult(sql, count < 1 ? "ok" : "update count: " + count, null);
             }
         } catch (SQLException e) {
@@ -368,7 +547,6 @@ public class TestScript extends TestBase {
     }
 
     private void writeResultSet(String sql, ResultSet rs) throws Exception {
-        boolean ordered = StringUtils.toLowerEnglish(sql).contains("order by");
         ResultSetMetaData meta = rs.getMetaData();
         int len = meta.getColumnCount();
         int[] max = new int[len];
@@ -392,9 +570,20 @@ public class TestScript extends TestBase {
             }
             head[i] = label;
         }
+        Boolean gotOrdered = null;
+        Statement st = rs.getStatement();
+        if (st instanceof JdbcPreparedStatement) {
+            CommandInterface ci = (CommandInterface) COMMAND.get(st);
+            if (ci instanceof CommandContainer) {
+                Prepared p = (Prepared) PREPARED.get(ci);
+                if (p instanceof Query) {
+                    gotOrdered = ((Query) p).hasOrder();
+                }
+            }
+        }
         rs.close();
         String line = readLine();
-        putBack = line;
+        putBack(line);
         if (line != null && line.startsWith(">> ")) {
             switch (result.size()) {
             case 0:
@@ -413,20 +602,52 @@ public class TestScript extends TestBase {
                 return;
             }
         }
+        Boolean ordered;
+        for (;;) {
+            line = readNextLine();
+            if (line == null) {
+                addWriteResultError("<row count>", "<eof>");
+                return;
+            }
+            putBack(line);
+            if (line.startsWith("> rows: ")) {
+                ordered = false;
+                break;
+            } else if (line.startsWith("> rows (ordered): ")) {
+                ordered = true;
+                break;
+            } else if (line.startsWith("> rows (partially ordered): ")) {
+                ordered = null;
+                break;
+            }
+        }
+        if (gotOrdered != null) {
+            if (ordered == null || ordered) {
+                if (!gotOrdered) {
+                    addWriteResultError("<ordered result set>", "<result set>");
+                }
+            } else {
+                if (gotOrdered) {
+                    addWriteResultError("<result set>", "<ordered result set>");
+                }
+            }
+        }
         writeResult(sql, format(head, max), null);
         writeResult(sql, format(null, max), null);
         String[] array = new String[result.size()];
         for (int i = 0; i < result.size(); i++) {
             array[i] = format(result.get(i), max);
         }
-        if (!ordered) {
+        if (!Boolean.TRUE.equals(ordered)) {
             sort(array);
         }
         int i = 0;
         for (; i < array.length; i++) {
             writeResult(sql, array[i], null);
         }
-        writeResult(sql, (ordered ? "rows (ordered): " : "rows: ") + i, null);
+        writeResult(sql,
+                (ordered != null ? ordered ? "rows (ordered): " : "rows: " : "rows (partially ordered): ") + i,
+                null);
     }
 
     private static String format(String[] row, int[] max) {
@@ -453,16 +674,31 @@ public class TestScript extends TestBase {
         return buff.toString();
     }
 
-    private void writeException(String sql, SQLException e) throws Exception {
-        writeResult(sql, "exception", e);
+    /** Convert the error code to a symbolic name from ErrorCode. */
+    private static final Map<Integer, String> ERROR_CODE_TO_NAME = new HashMap<>(256);
+    static {
+        try {
+            for (Field field : ErrorCode.class.getDeclaredFields()) {
+                if (field.getModifiers() == (Modifier.PUBLIC | Modifier.STATIC | Modifier.FINAL)
+                        && field.getAnnotation(Deprecated.class) == null) {
+                    ERROR_CODE_TO_NAME.put(field.getInt(null), field.getName());
+                }
+            }
+        } catch (IllegalAccessException ex) {
+            throw new RuntimeException(ex);
+        }
     }
 
-    private void writeResult(String sql, String s, SQLException e) throws Exception {
-        writeResult(sql, s, e, "> ");
+    private void writeException(String sql, SQLException ex) throws Exception {
+        writeResult(sql, "exception " + ERROR_CODE_TO_NAME.get(ex.getErrorCode()), ex);
     }
 
-    private void writeResult(String sql, String s, SQLException e, String prefix) throws Exception {
-        assertKnownException(sql, e);
+    private void writeResult(String sql, String s, SQLException ex) throws Exception {
+        writeResult(sql, s, ex, "> ");
+    }
+
+    private void writeResult(String sql, String s, SQLException ex, String prefix) throws Exception {
+        assertKnownException(sql, ex);
         s = (prefix + s).trim();
         String compare = readLine();
         if (compare != null && compare.startsWith(">")) {
@@ -470,27 +706,34 @@ public class TestScript extends TestBase {
                 if (reconnectOften && sql.toUpperCase().startsWith("EXPLAIN")) {
                     return;
                 }
-                errors.append(fileName).append('\n');
-                errors.append("line: ").append(outputLineNo).append('\n');
-                errors.append("exp: ").append(compare).append('\n');
-                errors.append("got: ").append(s).append('\n');
-                if (e != null) {
-                    TestBase.logError("script", e);
+                addWriteResultError(compare, s);
+                if (ex != null) {
+                    TestBase.logError("script", ex);
                 }
-                TestBase.logErrorMessage(errors.toString());
                 if (failFast) {
                     conn.close();
                     System.exit(1);
                 }
             }
         } else {
-            putBack = compare;
+            addWriteResultError("<nothing>", s);
+            if (compare != null) {
+                putBack(compare);
+            }
         }
         write(s);
     }
 
+    private void addWriteResultError(String expected, String got) {
+        int idx = errors.length();
+        errors.append(fileName).append('\n');
+        errors.append("line: ").append(in.getLineNumber()).append('\n');
+        errors.append("exp: ").append(expected).append('\n');
+        errors.append("got: ").append(got).append('\n');
+        TestBase.logErrorMessage(errors.substring(idx));
+    }
+
     private void write(String s) {
-        outputLineNo++;
         out.println(s);
     }
 
